@@ -3,14 +3,14 @@ use std::collections::HashMap;
 use datasize::DataSize;
 use serde::{Deserialize, Serialize};
 
-use super::{BlockProposerDeploySets, DeployInfo};
-use crate::types::{DeployHash, Timestamp};
+use super::{deploy_sets::PendingDeployInfo, BlockProposerDeploySets};
+use crate::types::DeployHash;
 
 /// State which is put to storage and loaded on initialization.
 #[derive(Serialize, Deserialize, Default, Debug, DataSize)]
 pub(crate) struct CachedState {
-    pub(super) pending_deploys: HashMap<DeployHash, (DeployInfo, Timestamp)>,
-    pub(super) pending_transfers: HashMap<DeployHash, (DeployInfo, Timestamp)>,
+    pub(super) pending_deploys: HashMap<DeployHash, PendingDeployInfo>,
+    pub(super) pending_transfers: HashMap<DeployHash, PendingDeployInfo>,
 }
 
 impl From<&BlockProposerDeploySets> for CachedState {
