@@ -6,6 +6,7 @@ use casper_execution_engine::shared::utils;
 const DEFAULT_MAX_GLOBAL_STATE_SIZE: usize = 805_306_368_000; // 750 GiB
 const DEFAULT_MAX_READERS: u32 = 512;
 const DEFAULT_MAX_QUERY_DEPTH: u64 = 5;
+const DEFAULT_MANUAL_SYNC_ENABLED: bool = true;
 
 /// Contract runtime configuration.
 #[derive(Clone, Copy, DataSize, Debug, Deserialize, Serialize)]
@@ -50,7 +51,8 @@ impl Config {
     }
 
     pub(crate) fn manual_sync_enabled(&self) -> bool {
-        self.enable_manual_sync.unwrap_or(false)
+        self.enable_manual_sync
+            .unwrap_or(DEFAULT_MANUAL_SYNC_ENABLED)
     }
 }
 
@@ -60,7 +62,7 @@ impl Default for Config {
             max_global_state_size: Some(DEFAULT_MAX_GLOBAL_STATE_SIZE),
             max_readers: Some(DEFAULT_MAX_READERS),
             max_query_depth: Some(DEFAULT_MAX_QUERY_DEPTH),
-            enable_manual_sync: Some(false),
+            enable_manual_sync: Some(DEFAULT_MANUAL_SYNC_ENABLED),
         }
     }
 }
