@@ -62,6 +62,15 @@ pub(crate) enum Error {
         prestate_hash: Digest,
         failure: DeployParameterFailure,
     },
+
+    /// The deploy received by the node from the client has expired.
+    #[error("deploy received by the node expired at {deploy_expiry_timestamp} with node's time at {current_node_timestamp}")]
+    ExpiredDeploy {
+        /// The timestamp when the deploy expires.
+        deploy_expiry_timestamp: Timestamp,
+        /// The timestamp when the node validated the expiry timestamp.
+        current_node_timestamp: Timestamp,
+    },
 }
 
 /// A representation of the way in which a deploy failed validation checks.
@@ -73,9 +82,6 @@ pub(crate) enum DeployParameterFailure {
     /// Nonexistent contract at hash.
     #[error("contract at {contract_hash} does not exist")]
     NonexistentContractAtHash { contract_hash: ContractHash },
-    /// Nonexistent contract at name.
-    #[error("contract named {name} does not exist in Account's NamedKeys")]
-    NonexistentContractAtName { name: String },
     /// Nonexistent contract entrypoint.
     #[error("contract does not have {entry_point}")]
     NonexistentContractEntryPoint { entry_point: String },
@@ -84,9 +90,6 @@ pub(crate) enum DeployParameterFailure {
     NonexistentContractPackageAtHash {
         contract_package_hash: ContractPackageHash,
     },
-    /// Contract Package does not exist.
-    #[error("contract package named {name} does not exist in Account's NamedKeys")]
-    NonexistentContractPackageAtName { name: String },
     /// Invalid contract at given version.
     #[error("invalid contract at version: {contract_version}")]
     InvalidContractAtVersion { contract_version: ContractVersion },
@@ -198,6 +201,24 @@ impl DeployAcceptor {
             );
         }
 
+        // We only perform expiry checks on deploys received from the client.
+        if source.from_client() {
+            let current_node_timestamp = Timestamp::now();
+            if deploy.header().expired(current_node_timestamp) {
+                let time_of_expiry = deploy.header().expires();
+                debug!(%deploy, "deploy has expired");
+                return self.handle_invalid_deploy_result(
+                    effect_builder,
+                    EventMetadata::new(deploy, source, maybe_responder),
+                    Error::ExpiredDeploy {
+                        deploy_expiry_timestamp: time_of_expiry,
+                        current_node_timestamp,
+                    },
+                    verification_start_timestamp,
+                );
+            }
+        }
+
         if self.verify_accounts {
             effect_builder
                 .get_highest_block_from_storage()
@@ -441,9 +462,15 @@ impl DeployAcceptor {
             | ExecutableDeployItem::StoredVersionedContractByName { .. } => (),
         }
 
-        let account_hash = event_metadata.deploy.header().account().to_account_hash();
         match payment.identifier() {
-            ExecutableDeployItemIdentifier::Module | ExecutableDeployItemIdentifier::Transfer => {
+            // We skip validation if the identifier is a named key, since that could yield a
+            // validation success at block X, then a validation failure at block X+1 (e.g. if the
+            // named key is deleted, or updated to point to an item which will fail subsequent
+            // validation).
+            ExecutableDeployItemIdentifier::Module
+            | ExecutableDeployItemIdentifier::Transfer
+            | ExecutableDeployItemIdentifier::Contract(ContractIdentifier::Name(_))
+            | ExecutableDeployItemIdentifier::Package(ContractPackageIdentifier::Name { .. }) => {
                 self.verify_session_logic(
                     effect_builder,
                     event_metadata,
@@ -451,44 +478,32 @@ impl DeployAcceptor {
                     verification_start_timestamp,
                 )
             }
-            ExecutableDeployItemIdentifier::Contract(contract_identifier) => {
-                let (query_key, path) = match &contract_identifier {
-                    ContractIdentifier::Name(name) => {
-                        let query_key: Key = account_hash.into();
-                        let path = vec![name.clone()];
-                        (query_key, path)
-                    }
-                    ContractIdentifier::Hash(contract_hash) => {
-                        let query_key: Key = (*contract_hash).into();
-                        (query_key, vec![])
-                    }
-                };
+            ExecutableDeployItemIdentifier::Contract(ContractIdentifier::Hash(contract_hash)) => {
+                let query_key = Key::from(contract_hash);
+                let path = vec![];
                 effect_builder
                     .get_contract_for_validation(prestate_hash, query_key, path)
                     .event(move |maybe_contract| Event::GetContractResult {
                         event_metadata,
                         prestate_hash,
                         is_payment: true,
-                        contract_identifier,
+                        contract_hash,
                         maybe_contract,
                         verification_start_timestamp,
                     })
             }
-            ExecutableDeployItemIdentifier::Package(contract_package_identifier) => {
-                let (query_key, path): (Key, Vec<String>) = match &contract_package_identifier {
-                    ContractPackageIdentifier::Name { name, .. } => {
-                        let query_key: Key = account_hash.into();
-                        let path = vec![name.clone()];
-                        (query_key, path)
-                    }
-                    ContractPackageIdentifier::Hash {
-                        contract_package_hash,
-                        ..
-                    } => {
-                        let query_key = (*contract_package_hash).into();
-                        (query_key, vec![])
-                    }
-                };
+            ExecutableDeployItemIdentifier::Package(
+                ref
+                contract_package_identifier
+                @
+                ContractPackageIdentifier::Hash {
+                    contract_package_hash,
+                    ..
+                },
+            ) => {
+                let query_key = Key::from(contract_package_hash);
+                let path = vec![];
+                let maybe_package_version = contract_package_identifier.version();
                 effect_builder
                     .get_contract_package_for_validation(prestate_hash, query_key, path)
                     .event(
@@ -496,7 +511,8 @@ impl DeployAcceptor {
                             event_metadata,
                             prestate_hash,
                             is_payment: true,
-                            contract_package_identifier,
+                            contract_package_hash,
+                            maybe_package_version,
                             maybe_contract_package,
                             verification_start_timestamp,
                         },
@@ -551,53 +567,47 @@ impl DeployAcceptor {
             | ExecutableDeployItem::StoredVersionedContractByName { .. } => (),
         }
 
-        let account_hash = event_metadata.deploy.header().account().to_account_hash();
         match session.identifier() {
-            ExecutableDeployItemIdentifier::Module | ExecutableDeployItemIdentifier::Transfer => {
+            // We skip validation if the identifier is a named key, since that could yield a
+            // validation success at block X, then a validation failure at block X+1 (e.g. if the
+            // named key is deleted, or updated to point to an item which will fail subsequent
+            // validation).
+            ExecutableDeployItemIdentifier::Module
+            | ExecutableDeployItemIdentifier::Transfer
+            | ExecutableDeployItemIdentifier::Contract(ContractIdentifier::Name(_))
+            | ExecutableDeployItemIdentifier::Package(ContractPackageIdentifier::Name { .. }) => {
                 self.validate_deploy_cryptography(
                     effect_builder,
                     event_metadata,
                     verification_start_timestamp,
                 )
             }
-            ExecutableDeployItemIdentifier::Contract(contract_identifier) => {
-                let (query_key, path) = match &contract_identifier {
-                    ContractIdentifier::Name(name) => {
-                        let query_key: Key = account_hash.into();
-                        let path = vec![name.clone()];
-                        (query_key, path)
-                    }
-                    ContractIdentifier::Hash(contract_hash) => {
-                        let query_key: Key = (*contract_hash).into();
-                        (query_key, vec![])
-                    }
-                };
+            ExecutableDeployItemIdentifier::Contract(ContractIdentifier::Hash(contract_hash)) => {
+                let query_key = Key::from(contract_hash);
+                let path = vec![];
                 effect_builder
                     .get_contract_for_validation(prestate_hash, query_key, path)
                     .event(move |maybe_contract| Event::GetContractResult {
                         event_metadata,
                         prestate_hash,
                         is_payment: false,
-                        contract_identifier,
+                        contract_hash,
                         maybe_contract,
                         verification_start_timestamp,
                     })
             }
-            ExecutableDeployItemIdentifier::Package(contract_package_identifier) => {
-                let (query_key, path): (Key, Vec<String>) = match &contract_package_identifier {
-                    ContractPackageIdentifier::Name { name, .. } => {
-                        let query_key: Key = account_hash.into();
-                        let path = vec![name.clone()];
-                        (query_key, path)
-                    }
-                    ContractPackageIdentifier::Hash {
-                        contract_package_hash,
-                        ..
-                    } => {
-                        let query_key = (*contract_package_hash).into();
-                        (query_key, vec![])
-                    }
-                };
+            ExecutableDeployItemIdentifier::Package(
+                ref
+                contract_package_identifier
+                @
+                ContractPackageIdentifier::Hash {
+                    contract_package_hash,
+                    ..
+                },
+            ) => {
+                let query_key = Key::from(contract_package_hash);
+                let path = vec![];
+                let maybe_package_version = contract_package_identifier.version();
                 effect_builder
                     .get_contract_package_for_validation(prestate_hash, query_key, path)
                     .event(
@@ -605,7 +615,8 @@ impl DeployAcceptor {
                             event_metadata,
                             prestate_hash,
                             is_payment: false,
-                            contract_package_identifier,
+                            contract_package_hash,
+                            maybe_package_version,
                             maybe_contract_package,
                             verification_start_timestamp,
                         },
@@ -622,7 +633,7 @@ impl DeployAcceptor {
         prestate_hash: Digest,
         is_payment: bool,
         entry_point: String,
-        contract_identifier: ContractIdentifier,
+        contract_hash: ContractHash,
         maybe_contract: Option<Contract>,
         verification_start_timestamp: Timestamp,
     ) -> Effects<Event> {
@@ -630,7 +641,7 @@ impl DeployAcceptor {
             if !contract.entry_points().has_entry_point(&entry_point) {
                 debug!(
                     ?entry_point,
-                    ?contract_identifier,
+                    ?contract_hash,
                     ?prestate_hash,
                     "missing entry point in contract"
                 );
@@ -660,17 +671,8 @@ impl DeployAcceptor {
             );
         }
 
-        let failure = match contract_identifier {
-            ContractIdentifier::Name(name) => {
-                debug!(?name, "nonexistent contract with name");
-                DeployParameterFailure::NonexistentContractAtName { name }
-            }
-            ContractIdentifier::Hash(contract_hash) => {
-                debug!(?contract_hash, "nonexistent contract with hash");
-                DeployParameterFailure::NonexistentContractAtHash { contract_hash }
-            }
-        };
-
+        debug!(?contract_hash, "nonexistent contract with hash");
+        let failure = DeployParameterFailure::NonexistentContractAtHash { contract_hash };
         let error = Error::InvalidDeployParameters {
             prestate_hash,
             failure,
@@ -690,29 +692,19 @@ impl DeployAcceptor {
         event_metadata: EventMetadata,
         prestate_hash: Digest,
         is_payment: bool,
-        contract_package_identifier: ContractPackageIdentifier,
+        contract_package_hash: ContractPackageHash,
+        maybe_package_version: Option<ContractVersion>,
         maybe_contract_package: Option<ContractPackage>,
         verification_start_timestamp: Timestamp,
     ) -> Effects<Event> {
         match maybe_contract_package {
             None => {
-                let failure = match contract_package_identifier {
-                    ContractPackageIdentifier::Name { name, .. } => {
-                        debug!(?name, "nonexistent contract package with name");
-                        DeployParameterFailure::NonexistentContractPackageAtName { name }
-                    }
-                    ContractPackageIdentifier::Hash {
-                        contract_package_hash,
-                        ..
-                    } => {
-                        debug!(
-                            ?contract_package_hash,
-                            "nonexistent contract package with hash"
-                        );
-                        DeployParameterFailure::NonexistentContractPackageAtHash {
-                            contract_package_hash,
-                        }
-                    }
+                debug!(
+                    ?contract_package_hash,
+                    "nonexistent contract package with hash"
+                );
+                let failure = DeployParameterFailure::NonexistentContractPackageAtHash {
+                    contract_package_hash,
                 };
                 let error = Error::InvalidDeployParameters {
                     prestate_hash,
@@ -725,7 +717,7 @@ impl DeployAcceptor {
                     verification_start_timestamp,
                 )
             }
-            Some(contract_package) => match contract_package_identifier.version() {
+            Some(contract_package) => match maybe_package_version {
                 Some(contract_version) => {
                     let contract_version_key = ContractVersionKey::new(
                         self.protocol_version.value().major,
@@ -740,7 +732,7 @@ impl DeployAcceptor {
                                     event_metadata,
                                     prestate_hash,
                                     is_payment,
-                                    contract_identifier: ContractIdentifier::Hash(contract_hash),
+                                    contract_hash,
                                     maybe_contract,
                                     verification_start_timestamp,
                                 })
@@ -762,8 +754,13 @@ impl DeployAcceptor {
                         }
                     }
                 }
+                // We continue to the next step in None case due to the subjective
+                // nature of global state.
                 None => {
                     if is_payment {
+                        // This function can be used to validate both session and payment
+                        // code. However, there is a call order where payment code must
+                        // be verified first before verifying session code.
                         self.verify_session_logic(
                             effect_builder,
                             event_metadata,
@@ -923,7 +920,7 @@ impl<REv: ReactorEventT> Component<REv> for DeployAcceptor {
                 event_metadata,
                 prestate_hash,
                 is_payment,
-                contract_identifier,
+                contract_hash,
                 maybe_contract,
                 verification_start_timestamp,
             } => {
@@ -939,7 +936,7 @@ impl<REv: ReactorEventT> Component<REv> for DeployAcceptor {
                     prestate_hash,
                     is_payment,
                     entry_point,
-                    contract_identifier,
+                    contract_hash,
                     maybe_contract,
                     verification_start_timestamp,
                 )
@@ -948,7 +945,8 @@ impl<REv: ReactorEventT> Component<REv> for DeployAcceptor {
                 event_metadata,
                 prestate_hash,
                 is_payment,
-                contract_package_identifier,
+                contract_package_hash,
+                maybe_package_version,
                 maybe_contract_package,
                 verification_start_timestamp,
             } => self.handle_get_contract_package_result(
@@ -956,7 +954,8 @@ impl<REv: ReactorEventT> Component<REv> for DeployAcceptor {
                 event_metadata,
                 prestate_hash,
                 is_payment,
-                contract_package_identifier,
+                contract_package_hash,
+                maybe_package_version,
                 maybe_contract_package,
                 verification_start_timestamp,
             ),
