@@ -125,6 +125,10 @@ enum TestScenario {
     FromPeerMissingAccount,
     FromPeerAccountWithInsufficientWeight,
     FromPeerAccountWithInvalidAssociatedKeys,
+    FromPeerCustomPaymentContract(ContractScenario),
+    FromPeerCustomPaymentContractPackage(ContractPackageScenario),
+    FromPeerSessionContract(ContractScenario),
+    FromPeerSessionContractPackage(ContractPackageScenario),
     FromClientInvalidDeploy,
     FromClientMissingAccount,
     FromClientInsufficientBalance,
@@ -133,10 +137,10 @@ enum TestScenario {
     FromClientAccountWithInsufficientWeight,
     FromClientAccountWithInvalidAssociatedKeys,
     AccountWithUnknownBalance,
-    DeployWithCustomPaymentContract(ContractScenario),
-    DeployWithCustomPaymentContractPackage(ContractPackageScenario),
-    DeployWithSessionContract(ContractScenario),
-    DeployWithSessionContractPackage(ContractPackageScenario),
+    FromClientCustomPaymentContract(ContractScenario),
+    FromClientCustomPaymentContractPackage(ContractPackageScenario),
+    FromClientSessionContract(ContractScenario),
+    FromClientSessionContractPackage(ContractPackageScenario),
     DeployWithNativeTransferInPayment,
     DeployWithEmptySessionModuleBytes,
     DeployWithoutPaymentAmount,
@@ -145,6 +149,8 @@ enum TestScenario {
     DeployWithoutTransferTarget,
     DeployWithoutTransferAmount,
     BalanceCheckForDeploySentByPeer,
+    ShouldNotAcceptExpiredDeploySentByClient,
+    ShouldAcceptExpiredDeploySentByPeer,
 }
 
 impl TestScenario {
@@ -156,7 +162,12 @@ impl TestScenario {
             | TestScenario::BalanceCheckForDeploySentByPeer
             | TestScenario::FromPeerMissingAccount
             | TestScenario::FromPeerAccountWithInsufficientWeight
-            | TestScenario::FromPeerAccountWithInvalidAssociatedKeys => {
+            | TestScenario::FromPeerAccountWithInvalidAssociatedKeys
+            | TestScenario::FromPeerCustomPaymentContract(_)
+            | TestScenario::FromPeerCustomPaymentContractPackage(_)
+            | TestScenario::FromPeerSessionContract(_)
+            | TestScenario::FromPeerSessionContractPackage(_)
+            | TestScenario::ShouldAcceptExpiredDeploySentByPeer => {
                 Source::Peer(NodeId::random(rng))
             }
             TestScenario::FromClientInvalidDeploy
@@ -172,12 +183,13 @@ impl TestScenario {
             | TestScenario::DeployWithMangledTransferAmount
             | TestScenario::DeployWithoutTransferAmount
             | TestScenario::DeployWithoutTransferTarget
-            | TestScenario::DeployWithCustomPaymentContract(_)
-            | TestScenario::DeployWithCustomPaymentContractPackage(_)
-            | TestScenario::DeployWithSessionContract(_)
-            | TestScenario::DeployWithSessionContractPackage(_)
+            | TestScenario::FromClientCustomPaymentContract(_)
+            | TestScenario::FromClientCustomPaymentContractPackage(_)
+            | TestScenario::FromClientSessionContract(_)
+            | TestScenario::FromClientSessionContractPackage(_)
             | TestScenario::DeployWithEmptySessionModuleBytes
-            | TestScenario::DeployWithNativeTransferInPayment => Source::Client,
+            | TestScenario::DeployWithNativeTransferInPayment
+            | TestScenario::ShouldNotAcceptExpiredDeploySentByClient => Source::Client,
         }
     }
 
@@ -217,7 +229,8 @@ impl TestScenario {
                 Deploy::random_with_mangled_transfer_amount(rng)
             }
 
-            TestScenario::DeployWithCustomPaymentContract(contract_scenario) => {
+            TestScenario::FromPeerCustomPaymentContract(contract_scenario)
+            | TestScenario::FromClientCustomPaymentContract(contract_scenario) => {
                 match contract_scenario {
                     ContractScenario::Valid | ContractScenario::MissingContractAtName => {
                         Deploy::random_with_valid_custom_payment_contract_by_name(rng)
@@ -230,7 +243,8 @@ impl TestScenario {
                     }
                 }
             }
-            TestScenario::DeployWithCustomPaymentContractPackage(contract_package_scenario) => {
+            TestScenario::FromPeerCustomPaymentContractPackage(contract_package_scenario)
+            | TestScenario::FromClientCustomPaymentContractPackage(contract_package_scenario) => {
                 match contract_package_scenario {
                     ContractPackageScenario::Valid
                     | ContractPackageScenario::MissingPackageAtName => {
@@ -244,7 +258,9 @@ impl TestScenario {
                     }
                 }
             }
-            TestScenario::DeployWithSessionContract(contract_scenario) => match contract_scenario {
+            TestScenario::FromPeerSessionContract(contract_scenario)
+            | TestScenario::FromClientSessionContract(contract_scenario) => match contract_scenario
+            {
                 ContractScenario::Valid | ContractScenario::MissingContractAtName => {
                     Deploy::random_with_valid_session_contract_by_name(rng)
                 }
@@ -255,7 +271,8 @@ impl TestScenario {
                     Deploy::random_with_missing_entry_point_in_session_contract(rng)
                 }
             },
-            TestScenario::DeployWithSessionContractPackage(contract_package_scenario) => {
+            TestScenario::FromPeerSessionContractPackage(contract_package_scenario)
+            | TestScenario::FromClientSessionContractPackage(contract_package_scenario) => {
                 match contract_package_scenario {
                     ContractPackageScenario::Valid
                     | ContractPackageScenario::MissingPackageAtName => {
@@ -275,6 +292,10 @@ impl TestScenario {
             TestScenario::DeployWithNativeTransferInPayment => {
                 Deploy::random_with_native_transfer_in_payment_logic(rng)
             }
+            TestScenario::ShouldAcceptExpiredDeploySentByPeer
+            | TestScenario::ShouldNotAcceptExpiredDeploySentByClient => {
+                Deploy::random_expired_deploy(rng)
+            }
         }
     }
 
@@ -286,7 +307,8 @@ impl TestScenario {
             | TestScenario::FromPeerAccountWithInsufficientWeight // account check skipped if from peer
             | TestScenario::FromPeerAccountWithInvalidAssociatedKeys // account check skipped if from peer
             | TestScenario::FromClientRepeatedValidDeploy
-            | TestScenario::FromClientValidDeploy => true,
+            | TestScenario::FromClientValidDeploy
+            | TestScenario::ShouldAcceptExpiredDeploySentByPeer=> true,
             TestScenario::FromPeerInvalidDeploy
             | TestScenario::FromClientInsufficientBalance
             | TestScenario::FromClientMissingAccount
@@ -301,20 +323,25 @@ impl TestScenario {
             | TestScenario::DeployWithMangledTransferAmount
             | TestScenario::DeployWithoutTransferAmount
             | TestScenario::DeployWithoutTransferTarget
-            | TestScenario::BalanceCheckForDeploySentByPeer => false,
-            TestScenario::DeployWithCustomPaymentContract(contract_scenario)
-            | TestScenario::DeployWithSessionContract(contract_scenario) => match contract_scenario
+            | TestScenario::BalanceCheckForDeploySentByPeer
+            | TestScenario::ShouldNotAcceptExpiredDeploySentByClient => false,
+            TestScenario::FromPeerCustomPaymentContract(contract_scenario)
+            | TestScenario::FromPeerSessionContract(contract_scenario)
+            | TestScenario::FromClientCustomPaymentContract(contract_scenario)
+            | TestScenario::FromClientSessionContract(contract_scenario) => match contract_scenario
             {
-                ContractScenario::Valid => true,
-                ContractScenario::MissingContractAtName
+                ContractScenario::Valid
+                | ContractScenario::MissingContractAtName => true,
                 | ContractScenario::MissingContractAtHash
                 | ContractScenario::MissingEntryPoint => false,
             },
-            TestScenario::DeployWithCustomPaymentContractPackage(contract_package_scenario)
-            | TestScenario::DeployWithSessionContractPackage(contract_package_scenario) => {
+            TestScenario::FromPeerCustomPaymentContractPackage(contract_package_scenario)
+            | TestScenario::FromPeerSessionContractPackage(contract_package_scenario)
+            | TestScenario::FromClientCustomPaymentContractPackage(contract_package_scenario)
+            | TestScenario::FromClientSessionContractPackage(contract_package_scenario) => {
                 match contract_package_scenario {
-                    ContractPackageScenario::Valid => true,
-                    ContractPackageScenario::MissingPackageAtName
+                    ContractPackageScenario::Valid
+                    | ContractPackageScenario::MissingPackageAtName => true,
                     | ContractPackageScenario::MissingPackageAtHash
                     | ContractPackageScenario::MissingContractVersion => false,
                 }
@@ -443,10 +470,16 @@ impl reactor::Reactor for Reactor {
                             }
                         } else {
                             match self.test_scenario {
-                                TestScenario::DeployWithCustomPaymentContractPackage(
+                                TestScenario::FromPeerCustomPaymentContractPackage(
                                     contract_package_scenario,
                                 )
-                                | TestScenario::DeployWithSessionContractPackage(
+                                | TestScenario::FromPeerSessionContractPackage(
+                                    contract_package_scenario,
+                                )
+                                | TestScenario::FromClientCustomPaymentContractPackage(
+                                    contract_package_scenario,
+                                )
+                                | TestScenario::FromClientSessionContractPackage(
                                     contract_package_scenario,
                                 ) => match contract_package_scenario {
                                     ContractPackageScenario::Valid
@@ -460,8 +493,10 @@ impl reactor::Reactor for Reactor {
                                     }
                                     _ => QueryResult::ValueNotFound(String::new()),
                                 },
-                                TestScenario::DeployWithSessionContract(contract_scenario)
-                                | TestScenario::DeployWithCustomPaymentContract(
+                                TestScenario::FromPeerSessionContract(contract_scenario)
+                                | TestScenario::FromPeerCustomPaymentContract(contract_scenario)
+                                | TestScenario::FromClientSessionContract(contract_scenario)
+                                | TestScenario::FromClientCustomPaymentContract(
                                     contract_scenario,
                                 ) => match contract_scenario {
                                     ContractScenario::Valid
@@ -476,8 +511,10 @@ impl reactor::Reactor for Reactor {
                         }
                     } else if let Key::Hash(_) = query_request.key() {
                         match self.test_scenario {
-                            TestScenario::DeployWithSessionContract(contract_scenario)
-                            | TestScenario::DeployWithCustomPaymentContract(contract_scenario) => {
+                            TestScenario::FromPeerSessionContract(contract_scenario)
+                            | TestScenario::FromPeerCustomPaymentContract(contract_scenario)
+                            | TestScenario::FromClientSessionContract(contract_scenario)
+                            | TestScenario::FromClientCustomPaymentContract(contract_scenario) => {
                                 match contract_scenario {
                                     ContractScenario::Valid
                                     | ContractScenario::MissingEntryPoint => QueryResult::Success {
@@ -490,10 +527,16 @@ impl reactor::Reactor for Reactor {
                                     }
                                 }
                             }
-                            TestScenario::DeployWithSessionContractPackage(
+                            TestScenario::FromPeerSessionContractPackage(
                                 contract_package_scenario,
                             )
-                            | TestScenario::DeployWithCustomPaymentContractPackage(
+                            | TestScenario::FromPeerCustomPaymentContractPackage(
+                                contract_package_scenario,
+                            )
+                            | TestScenario::FromClientSessionContractPackage(
+                                contract_package_scenario,
+                            )
+                            | TestScenario::FromClientCustomPaymentContractPackage(
                                 contract_package_scenario,
                             ) => match contract_package_scenario {
                                 ContractPackageScenario::Valid
@@ -638,7 +681,7 @@ async fn run_deploy_acceptor_without_timeout(
     let block = Box::new(Block::random(&mut rng));
     // Create a responder to assert that the block was successfully injected into storage.
     let (block_sender, block_receiver) = oneshot::channel();
-    let block_responder = Responder::create(block_sender);
+    let block_responder = Responder::without_shutdown(block_sender);
 
     runner
         .process_injected_effects(put_block_to_storage(block, block_responder))
@@ -653,7 +696,7 @@ async fn run_deploy_acceptor_without_timeout(
 
     // Create a responder to assert the validity of the deploy
     let (deploy_sender, deploy_receiver) = oneshot::channel();
-    let deploy_responder = Responder::create(deploy_sender);
+    let deploy_responder = Responder::without_shutdown(deploy_sender);
 
     // Create a deploy specific to the test scenario
     let deploy = test_scenario.deploy(&mut rng);
@@ -665,7 +708,7 @@ async fn run_deploy_acceptor_without_timeout(
         if test_scenario.is_repeated_deploy_case() {
             let injected_deploy = Box::new(deploy.clone());
             let (injected_sender, injected_receiver) = oneshot::channel();
-            let injected_responder = Responder::create(injected_sender);
+            let injected_responder = Responder::without_shutdown(injected_sender);
             runner
                 .process_injected_effects(put_deploy_to_storage(
                     injected_deploy,
@@ -682,7 +725,7 @@ async fn run_deploy_acceptor_without_timeout(
         if test_scenario == TestScenario::BalanceCheckForDeploySentByPeer {
             let fatal_deploy = Box::new(deploy.clone());
             let (deploy_sender, _) = oneshot::channel();
-            let deploy_responder = Responder::create(deploy_sender);
+            let deploy_responder = Responder::without_shutdown(deploy_sender);
             runner
                 .process_injected_effects(inject_balance_check_for_peer(
                     fatal_deploy,
@@ -722,7 +765,8 @@ async fn run_deploy_acceptor_without_timeout(
             | TestScenario::DeployWithMangledPaymentAmount
             | TestScenario::DeployWithMangledTransferAmount
             | TestScenario::DeployWithoutTransferTarget
-            | TestScenario::DeployWithoutTransferAmount => {
+            | TestScenario::DeployWithoutTransferAmount
+            | TestScenario::ShouldNotAcceptExpiredDeploySentByClient => {
                 matches!(
                     event,
                     Event::DeployAcceptorAnnouncement(DeployAcceptorAnnouncement::InvalidDeploy {
@@ -731,42 +775,45 @@ async fn run_deploy_acceptor_without_timeout(
                     })
                 )
             }
-            // Check that executable items with valid contracts are successfully stored.
-            // Conversely, ensure that invalid contracts will raise the invalid deploy
-            // announcement.
-            TestScenario::DeployWithCustomPaymentContract(contract_scenario)
-            | TestScenario::DeployWithSessionContract(contract_scenario) => match contract_scenario
+            // Check that executable items with valid contracts are successfully stored. Conversely,
+            // ensure that invalid contracts will raise the invalid deploy announcement.
+            TestScenario::FromPeerCustomPaymentContract(contract_scenario)
+            | TestScenario::FromPeerSessionContract(contract_scenario)
+            | TestScenario::FromClientCustomPaymentContract(contract_scenario)
+            | TestScenario::FromClientSessionContract(contract_scenario) => match contract_scenario
             {
-                ContractScenario::Valid => matches!(
+                ContractScenario::Valid | ContractScenario::MissingContractAtName => matches!(
                     event,
                     Event::DeployAcceptorAnnouncement(
                         DeployAcceptorAnnouncement::AcceptedNewDeploy { .. }
                     )
                 ),
-                ContractScenario::MissingContractAtHash
-                | ContractScenario::MissingContractAtName
-                | ContractScenario::MissingEntryPoint => matches!(
-                    event,
-                    Event::DeployAcceptorAnnouncement(
-                        DeployAcceptorAnnouncement::InvalidDeploy { .. }
+                ContractScenario::MissingContractAtHash | ContractScenario::MissingEntryPoint => {
+                    matches!(
+                        event,
+                        Event::DeployAcceptorAnnouncement(
+                            DeployAcceptorAnnouncement::InvalidDeploy { .. }
+                        )
                     )
-                ),
+                }
             },
             // Check that executable items with valid contract packages are successfully stored.
             // Conversely, ensure that invalid contract packages will raise the invalid deploy
             // announcement.
-            TestScenario::DeployWithCustomPaymentContractPackage(contract_package_scenario)
-            | TestScenario::DeployWithSessionContractPackage(contract_package_scenario) => {
+            TestScenario::FromPeerCustomPaymentContractPackage(contract_package_scenario)
+            | TestScenario::FromPeerSessionContractPackage(contract_package_scenario)
+            | TestScenario::FromClientCustomPaymentContractPackage(contract_package_scenario)
+            | TestScenario::FromClientSessionContractPackage(contract_package_scenario) => {
                 match contract_package_scenario {
-                    ContractPackageScenario::Valid => matches!(
+                    ContractPackageScenario::Valid
+                    | ContractPackageScenario::MissingPackageAtName => matches!(
                         event,
                         Event::DeployAcceptorAnnouncement(
                             DeployAcceptorAnnouncement::AcceptedNewDeploy { .. }
                         )
                     ),
                     ContractPackageScenario::MissingContractVersion
-                    | ContractPackageScenario::MissingPackageAtHash
-                    | ContractPackageScenario::MissingPackageAtName => matches!(
+                    | ContractPackageScenario::MissingPackageAtHash => matches!(
                         event,
                         Event::DeployAcceptorAnnouncement(
                             DeployAcceptorAnnouncement::InvalidDeploy { .. }
@@ -790,7 +837,8 @@ async fn run_deploy_acceptor_without_timeout(
             TestScenario::FromPeerValidDeploy
             | TestScenario::FromPeerMissingAccount
             | TestScenario::FromPeerAccountWithInvalidAssociatedKeys
-            | TestScenario::FromPeerAccountWithInsufficientWeight => {
+            | TestScenario::FromPeerAccountWithInsufficientWeight
+            | TestScenario::ShouldAcceptExpiredDeploySentByPeer => {
                 matches!(
                     event,
                     Event::DeployAcceptorAnnouncement(
@@ -987,30 +1035,24 @@ async fn should_accept_repeated_valid_deploy_from_client() {
 }
 
 #[tokio::test]
-async fn should_accept_deploy_with_valid_custom_payment() {
-    let test_scenario = TestScenario::DeployWithCustomPaymentContract(ContractScenario::Valid);
+async fn should_accept_deploy_with_valid_custom_payment_from_client() {
+    let test_scenario = TestScenario::FromClientCustomPaymentContract(ContractScenario::Valid);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(result.is_ok())
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_custom_payment_contract_by_name() {
+async fn should_accept_deploy_with_missing_custom_payment_contract_by_name_from_client() {
     let test_scenario =
-        TestScenario::DeployWithCustomPaymentContract(ContractScenario::MissingContractAtName);
+        TestScenario::FromClientCustomPaymentContract(ContractScenario::MissingContractAtName);
     let result = run_deploy_acceptor(test_scenario).await;
-    assert!(matches!(
-        result,
-        Err(super::Error::InvalidDeployParameters {
-            failure: DeployParameterFailure::NonexistentContractAtName { .. },
-            ..
-        })
-    ))
+    assert!(result.is_ok())
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_custom_payment_contract_by_hash() {
+async fn should_reject_deploy_with_missing_custom_payment_contract_by_hash_from_client() {
     let test_scenario =
-        TestScenario::DeployWithCustomPaymentContract(ContractScenario::MissingContractAtHash);
+        TestScenario::FromClientCustomPaymentContract(ContractScenario::MissingContractAtHash);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(matches!(
         result,
@@ -1022,9 +1064,9 @@ async fn should_reject_deploy_with_missing_custom_payment_contract_by_hash() {
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_entry_point_custom_payment() {
+async fn should_reject_deploy_with_missing_entry_point_custom_payment_from_client() {
     let test_scenario =
-        TestScenario::DeployWithCustomPaymentContract(ContractScenario::MissingEntryPoint);
+        TestScenario::FromClientCustomPaymentContract(ContractScenario::MissingEntryPoint);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(matches!(
         result,
@@ -1036,31 +1078,25 @@ async fn should_reject_deploy_with_missing_entry_point_custom_payment() {
 }
 
 #[tokio::test]
-async fn should_accept_deploy_with_valid_payment_contract_package_by_name() {
+async fn should_accept_deploy_with_valid_payment_contract_package_by_name_from_client() {
     let test_scenario =
-        TestScenario::DeployWithCustomPaymentContractPackage(ContractPackageScenario::Valid);
+        TestScenario::FromClientCustomPaymentContractPackage(ContractPackageScenario::Valid);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(result.is_ok())
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_payment_contract_package_at_name() {
-    let test_scenario = TestScenario::DeployWithCustomPaymentContractPackage(
+async fn should_accept_deploy_with_missing_payment_contract_package_at_name_from_client() {
+    let test_scenario = TestScenario::FromClientCustomPaymentContractPackage(
         ContractPackageScenario::MissingPackageAtName,
     );
     let result = run_deploy_acceptor(test_scenario).await;
-    assert!(matches!(
-        result,
-        Err(super::Error::InvalidDeployParameters {
-            failure: DeployParameterFailure::NonexistentContractPackageAtName { .. },
-            ..
-        })
-    ))
+    assert!(result.is_ok())
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_payment_contract_package_at_hash() {
-    let test_scenario = TestScenario::DeployWithCustomPaymentContractPackage(
+async fn should_reject_deploy_with_missing_payment_contract_package_at_hash_from_client() {
+    let test_scenario = TestScenario::FromClientCustomPaymentContractPackage(
         ContractPackageScenario::MissingPackageAtHash,
     );
     let result = run_deploy_acceptor(test_scenario).await;
@@ -1074,8 +1110,8 @@ async fn should_reject_deploy_with_missing_payment_contract_package_at_hash() {
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_version_in_payment_contract_package() {
-    let test_scenario = TestScenario::DeployWithCustomPaymentContractPackage(
+async fn should_reject_deploy_with_missing_version_in_payment_contract_package_from_client() {
+    let test_scenario = TestScenario::FromClientCustomPaymentContractPackage(
         ContractPackageScenario::MissingContractVersion,
     );
     let result = run_deploy_acceptor(test_scenario).await;
@@ -1089,16 +1125,24 @@ async fn should_reject_deploy_with_missing_version_in_payment_contract_package()
 }
 
 #[tokio::test]
-async fn should_accept_deploy_with_valid_session_contract() {
-    let test_scenario = TestScenario::DeployWithSessionContract(ContractScenario::Valid);
+async fn should_accept_deploy_with_valid_session_contract_from_client() {
+    let test_scenario = TestScenario::FromClientSessionContract(ContractScenario::Valid);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(result.is_ok())
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_session_contract_by_hash() {
+async fn should_accept_deploy_with_missing_session_contract_by_name_from_client() {
     let test_scenario =
-        TestScenario::DeployWithSessionContract(ContractScenario::MissingContractAtHash);
+        TestScenario::FromClientSessionContract(ContractScenario::MissingContractAtName);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_session_contract_by_hash_from_client() {
+    let test_scenario =
+        TestScenario::FromClientSessionContract(ContractScenario::MissingContractAtHash);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(matches!(
         result,
@@ -1110,23 +1154,9 @@ async fn should_reject_deploy_with_missing_session_contract_by_hash() {
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_session_contract_by_name() {
+async fn should_reject_deploy_with_missing_entry_point_in_session_contract_from_client() {
     let test_scenario =
-        TestScenario::DeployWithSessionContract(ContractScenario::MissingContractAtName);
-    let result = run_deploy_acceptor(test_scenario).await;
-    assert!(matches!(
-        result,
-        Err(super::Error::InvalidDeployParameters {
-            failure: DeployParameterFailure::NonexistentContractAtName { .. },
-            ..
-        })
-    ))
-}
-
-#[tokio::test]
-async fn should_reject_deploy_with_missing_entry_point_in_session_contract() {
-    let test_scenario =
-        TestScenario::DeployWithSessionContract(ContractScenario::MissingEntryPoint);
+        TestScenario::FromClientSessionContract(ContractScenario::MissingEntryPoint);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(matches!(
         result,
@@ -1138,31 +1168,25 @@ async fn should_reject_deploy_with_missing_entry_point_in_session_contract() {
 }
 
 #[tokio::test]
-async fn should_accept_deploy_with_valid_session_contract_package() {
+async fn should_accept_deploy_with_valid_session_contract_package_from_client() {
     let test_scenario =
-        TestScenario::DeployWithSessionContractPackage(ContractPackageScenario::Valid);
+        TestScenario::FromClientSessionContractPackage(ContractPackageScenario::Valid);
     let result = run_deploy_acceptor(test_scenario).await;
     assert!(result.is_ok())
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_session_contract_package_at_name() {
-    let test_scenario = TestScenario::DeployWithSessionContractPackage(
+async fn should_accept_deploy_with_missing_session_contract_package_at_name_from_client() {
+    let test_scenario = TestScenario::FromClientSessionContractPackage(
         ContractPackageScenario::MissingPackageAtName,
     );
     let result = run_deploy_acceptor(test_scenario).await;
-    assert!(matches!(
-        result,
-        Err(super::Error::InvalidDeployParameters {
-            failure: DeployParameterFailure::NonexistentContractPackageAtName { .. },
-            ..
-        })
-    ))
+    assert!(result.is_ok())
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_session_contract_package_at_hash() {
-    let test_scenario = TestScenario::DeployWithSessionContractPackage(
+async fn should_reject_deploy_with_missing_session_contract_package_at_hash_from_client() {
+    let test_scenario = TestScenario::FromClientSessionContractPackage(
         ContractPackageScenario::MissingPackageAtHash,
     );
     let result = run_deploy_acceptor(test_scenario).await;
@@ -1176,8 +1200,185 @@ async fn should_reject_deploy_with_missing_session_contract_package_at_hash() {
 }
 
 #[tokio::test]
-async fn should_reject_deploy_with_missing_version_in_session_contract_package() {
-    let test_scenario = TestScenario::DeployWithCustomPaymentContractPackage(
+async fn should_reject_deploy_with_missing_version_in_session_contract_package_from_client() {
+    let test_scenario = TestScenario::FromClientCustomPaymentContractPackage(
+        ContractPackageScenario::MissingContractVersion,
+    );
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::InvalidContractAtVersion { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_valid_custom_payment_from_peer() {
+    let test_scenario = TestScenario::FromPeerCustomPaymentContract(ContractScenario::Valid);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_missing_custom_payment_contract_by_name_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerCustomPaymentContract(ContractScenario::MissingContractAtName);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_custom_payment_contract_by_hash_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerCustomPaymentContract(ContractScenario::MissingContractAtHash);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::NonexistentContractAtHash { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_entry_point_custom_payment_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerCustomPaymentContract(ContractScenario::MissingEntryPoint);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::NonexistentContractEntryPoint { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_valid_payment_contract_package_by_name_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerCustomPaymentContractPackage(ContractPackageScenario::Valid);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_missing_payment_contract_package_at_name_from_peer() {
+    let test_scenario = TestScenario::FromPeerCustomPaymentContractPackage(
+        ContractPackageScenario::MissingPackageAtName,
+    );
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_payment_contract_package_at_hash_from_peer() {
+    let test_scenario = TestScenario::FromPeerCustomPaymentContractPackage(
+        ContractPackageScenario::MissingPackageAtHash,
+    );
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::NonexistentContractPackageAtHash { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_version_in_payment_contract_package_from_peer() {
+    let test_scenario = TestScenario::FromPeerCustomPaymentContractPackage(
+        ContractPackageScenario::MissingContractVersion,
+    );
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::InvalidContractAtVersion { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_valid_session_contract_from_peer() {
+    let test_scenario = TestScenario::FromPeerSessionContract(ContractScenario::Valid);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_missing_session_contract_by_name_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerSessionContract(ContractScenario::MissingContractAtName);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_session_contract_by_hash_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerSessionContract(ContractScenario::MissingContractAtHash);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::NonexistentContractAtHash { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_entry_point_in_session_contract_from_peer() {
+    let test_scenario = TestScenario::FromPeerSessionContract(ContractScenario::MissingEntryPoint);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::NonexistentContractEntryPoint { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_valid_session_contract_package_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerSessionContractPackage(ContractPackageScenario::Valid);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_accept_deploy_with_missing_session_contract_package_at_name_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerSessionContractPackage(ContractPackageScenario::MissingPackageAtName);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_session_contract_package_at_hash_from_peer() {
+    let test_scenario =
+        TestScenario::FromPeerSessionContractPackage(ContractPackageScenario::MissingPackageAtHash);
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(
+        result,
+        Err(super::Error::InvalidDeployParameters {
+            failure: DeployParameterFailure::NonexistentContractPackageAtHash { .. },
+            ..
+        })
+    ))
+}
+
+#[tokio::test]
+async fn should_reject_deploy_with_missing_version_in_session_contract_package_from_peer() {
+    let test_scenario = TestScenario::FromPeerCustomPaymentContractPackage(
         ContractPackageScenario::MissingContractVersion,
     );
     let result = run_deploy_acceptor(test_scenario).await;
@@ -1279,6 +1480,20 @@ async fn should_reject_deploy_with_mangled_transfer_amount() {
     ))
 }
 
+#[tokio::test]
+async fn should_reject_expired_deploy_from_client() {
+    let test_scenario = TestScenario::ShouldNotAcceptExpiredDeploySentByClient;
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(matches!(result, Err(super::Error::ExpiredDeploy { .. })))
+}
+
+#[tokio::test]
+async fn should_accept_expired_deploy_from_peer() {
+    let test_scenario = TestScenario::ShouldAcceptExpiredDeploySentByPeer;
+    let result = run_deploy_acceptor(test_scenario).await;
+    assert!(result.is_ok())
+}
+
 #[tokio::test]
 #[should_panic]
 async fn should_panic_when_balance_checking_for_deploy_sent_by_peer() {
