@@ -25,7 +25,7 @@ use crate::{
         EffectBuilder, EffectExt, Effects,
     },
     protocol::Message as NodeMessage,
-    types::{Deploy, DeployHash, Item, NodeId},
+    types::{Deploy, DeployHash, DeployWithFinalizedApprovals, Item, NodeId},
     utils::Source,
     NodeRng,
 };
@@ -80,6 +80,7 @@ pub(crate) fn get_deploy_from_storage<T: Item + 'static, REv: ReactorEventT<T>>(
                     .pop()
                     .unwrap()
                     .ok_or_else(|| String::from("failed to get deploy from storage"))
+                    .map(DeployWithFinalizedApprovals::into_naive)
             } else {
                 Err(String::from("expected a single result"))
             };
