@@ -18,7 +18,10 @@ use tokio::time;
 use tracing::debug;
 
 use casper_execution_engine::{
-    core::engine_state::DEFAULT_MAX_RUNTIME_CALL_STACK_HEIGHT,
+    core::engine_state::{
+        engine_config::{DEFAULT_MAX_DELEGATOR_SIZE_LIMIT, DEFAULT_MINIMUM_DELEGATION_AMOUNT},
+        DEFAULT_MAX_RUNTIME_CALL_STACK_HEIGHT,
+    },
     shared::{system_config::SystemConfig, wasm_config::WasmConfig},
 };
 use casper_types::ProtocolVersion;
@@ -52,6 +55,7 @@ use crate::{
 };
 
 const MAX_ASSOCIATED_KEYS: u32 = 100;
+const MAX_STORED_VALUE_SIZE: u32 = 8 * 1024 * 1024;
 
 /// Top-level event for the reactor.
 #[derive(Debug, From, Serialize)]
@@ -208,6 +212,9 @@ impl reactor::Reactor for Reactor {
             SystemConfig::default(),
             MAX_ASSOCIATED_KEYS,
             DEFAULT_MAX_RUNTIME_CALL_STACK_HEIGHT,
+            MAX_STORED_VALUE_SIZE,
+            DEFAULT_MAX_DELEGATOR_SIZE_LIMIT,
+            DEFAULT_MINIMUM_DELEGATION_AMOUNT,
             registry,
         )
         .unwrap();
