@@ -270,7 +270,7 @@ mod tests {
     mod ed25519 {
         use rand::Rng;
 
-        use casper_types::{checksummed_hex, ED25519_TAG};
+        use casper_types::ED25519_TAG;
 
         use super::*;
         use crate::crypto::AsymmetricKeyExt;
@@ -314,10 +314,9 @@ mod tests {
             const KNOWN_KEY_PEM: &str = r#"-----BEGIN PRIVATE KEY-----
 MC4CAQAwBQYDK2VwBCIEINTuctv5E1hK1bbY8fdp+K06/nwoy/HU++CXqI9EdVhC
 -----END PRIVATE KEY-----"#;
-            let key_bytes = checksummed_hex::decode(
-                "d4ee72dbf913584ad5b6d8f1f769f8ad3afe7c28cbf1d4fbe097a88f44755842",
-            )
-            .unwrap();
+            let key_bytes =
+                base16::decode("d4ee72dbf913584ad5b6d8f1f769f8ad3afe7c28cbf1d4fbe097a88f44755842")
+                    .unwrap();
             let expected_key = SecretKey::ed25519_from_bytes(key_bytes).unwrap();
             super::known_secret_key_to_pem(&expected_key, KNOWN_KEY_PEM, ED25519_TAG);
         }
@@ -563,10 +562,9 @@ MHQCAQEEIL3fqaMKAfXSK1D2PnVVbZlZ7jTv133nukq4+95s6kmcoAcGBSuBBAAK
 oUQDQgAEQI6VJjFv0fje9IDdRbLMcv/XMnccnOtdkv+kBR5u4ISEAkuc2TFWQHX0
 Yj9oTB9fx9+vvQdxJOhMtu46kGo0Uw==
 -----END EC PRIVATE KEY-----"#;
-            let key_bytes = checksummed_hex::decode(
-                "bddfa9a30a01f5d22b50f63e75556d9959ee34efd77de7ba4ab8fbde6cea499c",
-            )
-            .unwrap();
+            let key_bytes =
+                base16::decode("bddfa9a30a01f5d22b50f63e75556d9959ee34efd77de7ba4ab8fbde6cea499c")
+                    .unwrap();
             let expected_key = SecretKey::secp256k1_from_bytes(key_bytes).unwrap();
             super::known_secret_key_to_pem(&expected_key, KNOWN_KEY_PEM, SECP256K1_TAG);
         }
@@ -821,13 +819,13 @@ kv+kBR5u4ISEAkuc2TFWQHX0Yj9oTB9fx9+vvQdxJOhMtu46kGo0Uw==
         // compressed.
         let uncompressed_hex = {
             let tag_bytes = vec![0x02u8];
-            checksummed_hex::encode(&tag_bytes)
-                + &checksummed_hex::encode(&secp256k1_public_key.to_encoded_point(false).as_bytes())
+            base16::encode_lower(&tag_bytes)
+                + &base16::encode_lower(&secp256k1_public_key.to_encoded_point(false).as_bytes())
         };
 
         format!(
             "02{}",
-            checksummed_hex::encode(secp256k1_public_key.to_encoded_point(false).as_bytes())
+            base16::encode_lower(secp256k1_public_key.to_encoded_point(false).as_bytes())
                 .to_lowercase()
         );
         let from_uncompressed_hex = PublicKey::from_hex(&uncompressed_hex).unwrap();
