@@ -30,7 +30,8 @@ pub use chainspec::Chainspec;
 pub use datasize::DataSize;
 pub use deploy::{
     Approval, Deploy, DeployConfigurationFailure, DeployHash, DeployHeader, DeployMetadata,
-    DeployOrTransferHash, Error as DeployError, ExcessiveSizeError as ExcessiveSizeDeployError,
+    DeployOrTransferHash, DeployWithApprovals, DeployWithFinalizedApprovals, Error as DeployError,
+    ExcessiveSizeError as ExcessiveSizeDeployError, FinalizedApprovals,
 };
 pub use error::BlockValidationError;
 pub use exit_code::ExitCode;
