@@ -30,6 +30,7 @@ pub struct DeployConfig {
     pub(crate) max_deploy_size: u32,
     pub(crate) block_max_deploy_count: u32,
     pub(crate) block_max_transfer_count: u32,
+    pub(crate) block_max_approval_count: u32,
     pub(crate) block_gas_limit: u64,
     pub(crate) payment_args_max_length: u32,
     pub(crate) session_args_max_length: u32,
@@ -47,6 +48,7 @@ impl DeployConfig {
         let max_deploy_size = rng.gen_range(100_000..1_000_000);
         let block_max_deploy_count = rng.gen();
         let block_max_transfer_count = rng.gen();
+        let block_max_approval_count = rng.gen();
         let block_gas_limit = rng.gen_range(100_000_000_000..1_000_000_000_000_000);
         let payment_args_max_length = rng.gen();
         let session_args_max_length = rng.gen();
@@ -61,6 +63,7 @@ impl DeployConfig {
             max_deploy_size,
             block_max_deploy_count,
             block_max_transfer_count,
+            block_max_approval_count,
             block_gas_limit,
             payment_args_max_length,
             session_args_max_length,
@@ -80,6 +83,7 @@ impl Default for DeployConfig {
             max_deploy_size: 1_048_576,
             block_max_deploy_count: 10,
             block_max_transfer_count: 1000,
+            block_max_approval_count: 2600,
             block_gas_limit: 10_000_000_000_000,
             payment_args_max_length: 1024,
             session_args_max_length: 1024,
@@ -98,6 +102,7 @@ impl ToBytes for DeployConfig {
         buffer.extend(self.max_deploy_size.to_bytes()?);
         buffer.extend(self.block_max_deploy_count.to_bytes()?);
         buffer.extend(self.block_max_transfer_count.to_bytes()?);
+        buffer.extend(self.block_max_approval_count.to_bytes()?);
         buffer.extend(self.block_gas_limit.to_bytes()?);
         buffer.extend(self.payment_args_max_length.to_bytes()?);
         buffer.extend(self.session_args_max_length.to_bytes()?);
@@ -113,6 +118,7 @@ impl ToBytes for DeployConfig {
             + self.max_deploy_size.serialized_length()
             + self.block_max_deploy_count.serialized_length()
             + self.block_max_transfer_count.serialized_length()
+            + self.block_max_approval_count.serialized_length()
             + self.block_gas_limit.serialized_length()
             + self.payment_args_max_length.serialized_length()
             + self.session_args_max_length.serialized_length()
@@ -130,6 +136,7 @@ impl FromBytes for DeployConfig {
         let (max_deploy_size, remainder) = u32::from_bytes(remainder)?;
         let (block_max_deploy_count, remainder) = u32::from_bytes(remainder)?;
         let (block_max_transfer_count, remainder) = u32::from_bytes(remainder)?;
+        let (block_max_approval_count, remainder) = u32::from_bytes(remainder)?;
         let (block_gas_limit, remainder) = u64::from_bytes(remainder)?;
         let (payment_args_max_length, remainder) = u32::from_bytes(remainder)?;
         let (session_args_max_length, remainder) = u32::from_bytes(remainder)?;
@@ -142,6 +149,7 @@ impl FromBytes for DeployConfig {
             max_deploy_size,
             block_max_deploy_count,
             block_max_transfer_count,
+            block_max_approval_count,
             block_gas_limit,
             payment_args_max_length,
             session_args_max_length,
